from typing import Callable, Generic, TypeVar

from .context import *

__all__ = ('AtOf',)


K = TypeVar('K')
V = TypeVar('V')


class AtOf(Generic[K, V]):
    def __call__(self) -> tuple[Callable[[K], Callable[[V], V]], Callable[[K], V]]:
        bucket: dict[K, V] = {}

        def at(key: K) -> Callable[[V], V]:
            def wrap(value: V) -> V:
                bucket[key] = value
                return value
            return wrap

        def of(key: K) -> V:
            try:
                return bucket[key]
            except IndexError:
                raise Implicit

        return at, of
