from io import StringIO
from typing import Awaitable, Callable, Optional, Union

import discord

__all__ = ('usertype', 'Context', 'escape', 'command_type', 'Explicit', 'Implicit')

usertype = Union[discord.abc.User, discord.user.BaseUser, discord.Member, discord.User]


class Context:
    def __init__(self, message: discord.Message, client: discord.Client):
        self.message: discord.Message = message
        self.client: discord.Client = client
        self.channel: discord.abc.Messageable = message.channel
        self.author: usertype = message.author
        self.content: str = message.content
        self.member: Optional[discord.Member] = message.author if isinstance(message.author, discord.Member) else None
        self.guild: Optional[discord.Guild] = None if self.member is None else self.member.guild

    async def reply(self, content=None, **kwargs) -> discord.Message:
        return await self.message.reply(content, mention_author=False, **kwargs)

    async def long(self, s: str):
        resio = StringIO(s)
        res = ''
        for line in resio:
            if len(res) + len(line) < 2000:
                res += line
            else:
                await self.reply(res)
                res = line
        if res:
            await self.reply(res)


ESCAPED = '`_*\'"\\'


def escape(s: str):
    res = StringIO()
    for c in s:
        if c in ESCAPED:
            c = '\\' + c
        res.write(c)
    return res.getvalue()


command_type = Callable[[Context, list[str]], Awaitable[None]]


class Explicit(Exception):
    def __init__(self, msg: str):
        self.msg = msg
        super().__init__(msg)


class Implicit(Exception):
    pass
